@@ -22,19 +22,21 @@
 ##
 ## 12/2/2003
 ## Revised to accommodate new spec 7/20/2004
+## Minor bug fix regarding std.mean 6/25/2004
 ##
 ##########################################################################
 
 "MCMCmixfactanal" <-
   function(x, factors, lambda.constraints=list(),
-           data=parent.environment(), burnin = 1000, mcmc = 20000,
-           thin=1, tune=NA, verbose = FALSE, seed = NA,
+           data=parent.frame(), burnin = 1000, mcmc = 20000,
+           thin=1, tune=NA, verbose = 0, seed = NA,
            lambda.start = NA, psi.start=NA,
            l0=0, L0=0, a0=0.001, b0=0.001,
            store.lambda=TRUE, store.scores=FALSE,
            std.mean=TRUE, std.var=TRUE, ... ) {
     
     call <- match.call()
+    echo.name <- NULL
     mt <- terms(x, data=data)
     if (attr(mt, "response") > 0) 
       stop("Response not allowed in formula in MCMCmixfactanal().\n")
@@ -43,9 +45,11 @@
     mf$factors <- mf$lambda.constraints <- mf$burnin <- mf$mcmc <- NULL
     mf$thin <- mf$tune <- mf$verbose <- mf$seed <- NULL
     mf$lambda.start <- mf$l0 <- mf$L0 <- mf$a0 <- mf$b0 <- NULL
-    mf$store.lambda <- mf$store.scores <- mf$std.var <- mf$... <- NULL
+    mf$store.lambda <- mf$store.scores <- mf$std.mean <- NULL
+    mf$std.var <- mf$... <- NULL
     mf$drop.unused.levels <- TRUE
     mf[[1]] <- as.name("model.frame")
+    mf$na.action <- 'na.pass'
     mf <- eval(mf, sys.frame(sys.parent()))
     attributes(mt)$intercept <- 0
     Xterm.length <- length(attr(mt, "variables"))
@@ -60,12 +64,13 @@
         ncat[i] <- -999
         X[is.na(X[,i]), i] <- -999
       }
-      else if (is.ordered(X[,i])){
-        ncat[i] <- nlevels(X[,i])      
-        X[,i] <- as.integer(X[,i])
-        X[is.na(X[,i]), i] <- -999
-      }
-      else {
+   else if (is.ordered(X[, i])) {
+     ncat[i] <- nlevels(X[, i])
+     temp <- as.integer(X[,i])
+     temp <- ifelse(is.na(X[,i]) | (X[,i] == "<NA>"), -999, temp)
+     X[, i] <- temp
+   }
+   else {
         stop("Manifest variable ", dimnames(X)[[2]][i],
              " neither ordered factor nor numeric variable.\n")
       }
@@ -90,7 +95,7 @@
     if (std.var){
       for (i in 1:K){
         if (ncat[i] == -999){
-          X[,i] <- (X[,i])/sd(X[,i])
+          X[,i] <- (X[,i] - mean(X[,i]))/sd(X[,i]) + mean(X[,i])
         }
       }
     }
@@ -226,6 +231,7 @@
         gamma[2,i] <- 0
         gamma[3:ncat[i],i] <- (polr.out$zeta[2:(ncat[i]-1)] -
                                polr.out$zeta[1])*.588
+
         gamma[ncat[i]+1,i] <- 300
       }
     }
@@ -238,7 +244,7 @@
       }
     }
           
-    # define holder for posterior density sample
+    # define holder for posterior sample
     if (store.scores == FALSE && store.lambda == FALSE){
       sample <- matrix(data=0, mcmc/thin, length(gamma)+K)
     }
@@ -252,8 +258,11 @@
       sample <- matrix(data=0, mcmc/thin, K*(factors+1)+(factors+1)*N +
                        length(gamma)+K)
     }
-    
+
+    accepts <- matrix(0, K, 1)
+
     # Call the C++ code to do the real work
+    posterior <- NULL
     posterior <- .C("mixfactanalpost",
                     samdata = as.double(sample),
                     samrow = as.integer(nrow(sample)),
@@ -301,17 +310,23 @@
                     b0col = as.integer(ncol(b0)),
                     storelambda = as.integer(store.lambda),
                     storescores = as.integer(store.scores),
-                    accepts = as.integer(0),
+                    accepts = as.integer(accepts),
+                    acceptsrow = as.integer(nrow(accepts)),
+                    acceptscol = as.integer(ncol(accepts)),
                     PACKAGE="MCMCpack"
                     )
 
-    cat(" overall acceptance rate = ",
-        posterior$accepts / ((posterior$burnin+posterior$mcmc)*n.ord.ge3), "\n")
-
-    
+    accepts <- matrix(posterior$accepts, posterior$acceptsrow,
+                      posterior$acceptscol, byrow=TRUE)
+    rownames(accepts) <- X.names
+    colnames(accepts) <- ""
+    cat("\n\nAcceptance rates:\n")
+    print(t(accepts) / (posterior$burnin+posterior$mcmc), digits=2,
+          width=6)      
+        
     # put together matrix and build MCMC object to return
     sample <- matrix(posterior$samdata, posterior$samrow, posterior$samcol,
-                     byrow=TRUE)
+                     byrow=FALSE)
     output <- mcmc(data=sample,start=1, end=mcmc, thin=thin)
     
     par.names <- NULL
@@ -345,16 +360,16 @@
     output.df <- as.data.frame(as.matrix(output))
     output.var <- diag(var(output.df))
     output.df <- output.df[,output.var != 0]
-    output <- mcmc(as.matrix(output.df), start=1, end=mcmc, thin=thin)
+    output <- mcmc(as.matrix(output.df), start=burnin+1, end=burnin+mcmc,
+                   thin=thin)
     
     # add constraint info so this isn't lost
     attr(output, "constraints") <- lambda.constraints
     attr(output, "n.manifest") <- K
     attr(output, "n.factors") <- factors
-    attr(output, "accept.rate") <- posterior$accepts /
-      ((posterior$burnin+posterior$mcmc)*n.ord.ge3)
+    attr(output, "accept.rates") <- t(accepts) / (posterior$burnin+posterior$mcmc)
       attr(output,"title") <-
-        "MCMCpack Mixed Data Factor Analysis Posterior Density Sample"
+        "MCMCpack Mixed Data Factor Analysis Posterior Sample"
     
     return(output)
     
